@@ -110,16 +110,6 @@ function onClick(e, id, item, subItem) {
 		toolbar = w2ui['presentation-toolbar'];
 		item = toolbar.get(id);
 	}
-	else if (item && subItem)
-	{
-		var command = {
-			'StatusBarFunc': {
-				type: 'unsigned short',
-				value: subItem.func
-			}
-		};
-		map.sendUnoCommand('.uno:StatusBarFunc', command);
-	}
 	else {
 		throw new Error('unknown id: ' + id);
 	}
@@ -245,6 +235,15 @@ function onClick(e, id, item, subItem) {
 		w2ui['formulabar'].hide('acceptformula', 'cancelformula');
 		w2ui['formulabar'].show('sum', 'function');
 	}
+	else if (id.startsWith('StateTableCellMenu') && subItem) {
+		var command = {
+			'StatusBarFunc': {
+				type: 'unsigned short',
+				value: subItem.func
+			}
+		};
+		map.sendUnoCommand('.uno:StatusBarFunc', command);
+	}
 }
 
 function insertTable() {
@@ -296,68 +295,6 @@ function insertTable() {
 	}, '.col');
 }
 
-var basicShapes = [
-	{img: 'basicshapes_rectangle', uno: 'BasicShapes.rectangle'},
-	{img: 'basicshapes_round-rectangle', uno: 'BasicShapes.round-rectangle'},
-	{img: 'basicshapes_quadrat', uno: 'BasicShapes.quadrat'},
-	{img: 'basicshapes_round-quadrat', uno: 'BasicShapes.round-quadrat'},
-	{img: 'basicshapes_circle', uno: 'BasicShapes.circle'},
-	{img: 'basicshapes_ellipse', uno: 'BasicShapes.ellipse'},
-
-	{img: 'basicshapes_circle-pie', uno: 'BasicShapes.circle-pie'},
-	{img: 'basicshapes_isosceles-triangle', uno: 'BasicShapes.isosceles-triangle'},
-	{img: 'basicshapes_right-triangle', uno: 'BasicShapes.right-triangle'},
-	{img: 'basicshapes_trapezoid', uno: 'BasicShapes.trapezoid'},
-	{img: 'basicshapes_diamond', uno: 'BasicShapes.diamong'},
-	{img: 'basicshapes_parallelogram', uno: 'BasicShapes.parallelogram'},
-
-	{img: 'basicshapes_pentagon', uno: 'BasicShapes.pentagon'},
-	{img: 'basicshapes_hexagon', uno: 'BasicShapes.hexagon'},
-	{img: 'basicshapes_octagon', uno: 'BasicShapes.octagon'},
-	{img: 'basicshapes_cross', uno: 'BasicShapes.cross'},
-	{img: 'basicshapes_ring', uno: 'BasicShapes.ring'},
-	{img: 'basicshapes_block-arc', uno: 'BasicShapes.block-arc'},
-
-	{img: 'basicshapes_can', uno: 'BasicShapes.can'},
-	{img: 'basicshapes_cube', uno: 'BasicShapes.cube'},
-	{img: 'basicshapes_paper', uno: 'BasicShapes.paper'},
-	{img: 'basicshapes_frame', uno: 'BasicShapes.frame'}
-];
-
-function insertShapes() {
-	var width = 6;
-	var $grid = $('.insertshape-grid');
-
-	var rows = Math.ceil(basicShapes.length / width);
-	var idx = 0;
-
-	if ($grid.children().size() == rows)
-		return;
-
-	for (var r = 0; r < rows; r++) {
-		var $row = $('<div/>').addClass('row');
-		$grid.append($row);
-		for (var c = 0; c < width; c++) {
-			if (idx >= basicShapes.length) {
-				break;
-			}
-			var shape = basicShapes[idx++];
-			var $col = $('<div/>').addClass('col w2ui-icon').addClass(shape.img);
-			$col.data('uno', shape.uno);
-			$row.append($col);
-		}
-
-		if (idx >= basicShapes.length)
-			break;
-	}
-
-	$grid.on({
-		click: function(e) {
-			map.sendUnoCommand('.uno:' + $(e.target).data().uno);
-		}
-	});
-}
-
 function onColorPick(id, color) {
 	if (map.getPermission() !== 'edit' || color === undefined) {
 		return;
@@ -451,8 +388,6 @@ function createToolbar() {
 			{type: 'break', id: 'incdecindent'},
 			{type: 'drop',  id: 'inserttable',  img: 'inserttable', hint: _('Insert table'), overlay: {onShow: insertTable},
 			 html: '<div id="inserttable-wrapper"><div id="inserttable-popup" class="inserttable-pop ui-widget ui-widget-content ui-corner-all"><div class="inserttable-grid"></div><div id="inserttable-status" class="loleaflet-font" style="padding: 5px;"><br/></div></div></div>'},
-			{type: 'drop',  id: 'insertshapes',  img: 'insertshapes', hint: _('Insert shapes'), overlay: {onShow: insertShapes},
-			 html: '<div id="insertshape-wrapper"><div id="insertshape-popup" class="insertshape-pop ui-widget ui-widget-content ui-corner-all"><div class="insertshape-grid"></div></div></div>'},
 			{type: 'button',  id: 'insertobjectchart',  img: 'insertobjectchart', hint: _UNO('.uno:InsertObjectChart', '', true), uno: 'InsertObjectChart'},
 			{type: 'button',  id: 'insertannotation', img: 'annotation', hint: _UNO('.uno:InsertAnnotation', '', true)},
 			{type: 'button',  id: 'insertgraphic',  img: 'insertgraphic', hint: _UNO('.uno:InsertGraphic', '', true)},
@@ -484,8 +419,6 @@ function createToolbar() {
 			updateCommandValues();
 
 			insertTable();
-
-			insertShapes();
 		}
 	});
 
