@@ -291,7 +291,7 @@ L.TileLayer = L.GridLayer.extend({
 		for (var key in this._selectionHandles) {
 			this._selectionHandles[key].on('drag dragend', this._onSelectionHandleDrag, this);
 		}
-		this._map._clipboardContainer.focus(true);
+		this._map.focus();
 
 		map.setPermission(this.options.permission);
 
@@ -1554,6 +1554,22 @@ L.TileLayer = L.GridLayer.extend({
 		this._onUpdateCursor();
 	},
 
+	_updateCursorPos: function () {
+		var pixBounds = L.bounds(this._map.latLngToLayerPoint(this._visibleCursor.getSouthWest()),
+			this._map.latLngToLayerPoint(this._visibleCursor.getNorthEast()));
+		var cursorPos = this._visibleCursor.getNorthWest();
+
+		if (!this._cursorMarker) {
+			this._cursorMarker = L.cursor(cursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())), {blink: true});
+		}
+		else {
+			this._cursorMarker.setLatLng(cursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())));
+		}
+
+		this._map.addLayer(this._cursorMarker);
+		return this._visibleCursor.getNorthWest();
+	},
+
 	// Update cursor layer (blinking cursor).
 	_onUpdateCursor: function (scroll) {
 		var cursorPos = this._visibleCursor.getNorthWest();
@@ -1586,23 +1602,11 @@ L.TileLayer = L.GridLayer.extend({
 	// the state of the document (if the falgs are set)
 	_updateCursorAndOverlay: function (/*update*/) {
 		if (this._map._permission === 'edit'
+		&& this._map._clipboardContainer._textArea === document.activeElement
 		&& this._isCursorVisible
 		&& this._isCursorOverlayVisible
 		&& !this._isEmptyRectangle(this._visibleCursor)) {
-
-			var pixBounds = L.bounds(this._map.latLngToLayerPoint(this._visibleCursor.getSouthWest()),
-									 this._map.latLngToLayerPoint(this._visibleCursor.getNorthEast()));
-
-			var cursorPos = this._visibleCursor.getNorthWest();
-
-			if (!this._cursorMarker) {
-				this._cursorMarker = L.cursor(cursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())), {blink: true});
-			}
-			else {
-				this._cursorMarker.setLatLng(cursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())));
-			}
-			this._map.addLayer(this._cursorMarker);
-			this._map._clipboardContainer.setLatLng(this._visibleCursor.getNorthWest());
+			this._updateCursorPos();
 		}
 		else if (this._cursorMarker) {
 			this._map.removeLayer(this._cursorMarker);
@@ -1786,7 +1790,7 @@ L.TileLayer = L.GridLayer.extend({
 		}
 		if (e.type === 'dragend') {
 			e.target.isDragged = false;
-			this._map._clipboardContainer.focus(true);
+			this._map.focus();
 			this._map.fire('scrollvelocity', {vx: 0, vy: 0});
 		}
 
@@ -2157,20 +2161,22 @@ L.TileLayer = L.GridLayer.extend({
 	},
 
 	_fitWidthZoom: function (e, maxZoom) {
-		var size = e ? e.newSize : this._map.getSize();
-		var widthTwips = size.x * this._map.options.tileWidthTwips / this._tileSize;
-		maxZoom = maxZoom ? maxZoom : this._map.getZoom();
+		if (isNaN(this._docWidthTwips)) { return; }
+		var oldSize = e ? e.oldSize : this._map.getSize();
+		var newSize = e ? e.newSize : this._map.getSize();
+		if (newSize.x - oldSize.x === 0) { return; }
 
+		var widthTwips = newSize.x * this._map.options.tileWidthTwips / this._tileSize;
+		var ratio = widthTwips / this._docWidthTwips;
+
+		maxZoom = maxZoom ? maxZoom : this._map.options.zoom;
 		// 'fit width zoom' has no use in spreadsheets, ignore it there
 		if (this._docType !== 'spreadsheet') {
 			var crsScale = this._map.options.crs.scale(1);
-			if (this._docWidthTwips > 0)
-			{
-				var ratio = widthTwips / this._docWidthTwips;
-				var zoom = this._map.getZoom() + Math.floor(Math.log(ratio) / Math.log(crsScale));
+			var zoom = 10 + Math.floor(Math.log(ratio) / Math.log(crsScale));
 
-				zoom = Math.max(1, zoom);
-				zoom = Math.min(maxZoom, zoom);
+			zoom = Math.min(maxZoom, Math.max(1, zoom));
+			if (this._docWidthTwips * this._map.getZoomScale(zoom, 10) < widthTwips) {
 				this._map.setZoom(zoom, {animate: false});
 			}
 		}
