@@ -12,6 +12,7 @@ L.Map = L.Evented.extend({
 		zoom: 10,
 		minZoom: 1,
 		maxZoom: 20,
+		maxBounds: L.latLngBounds([0, 0], [-100, 100]),
 		fadeAnimation: false, // Not useful for typing.
 		trackResize: true,
 		markerZoomAnimation: true,
@@ -330,25 +331,14 @@ L.Map = L.Evented.extend({
 		return this.fire('moveend');
 	},
 
-	setMaxBounds: function (bounds, options) {
+	setMaxBounds: function (bounds) {
 		bounds = L.latLngBounds(bounds);
 
 		this.options.maxBounds = bounds;
-		options = options || {};
-
-		if (!bounds) {
-			return this.off('moveend', this._panInsideMaxBounds);
-		}
 
 		if (this._loaded) {
-			this._panInsideMaxBounds();
+			this.panInsideBounds(this.options.maxBounds);
 		}
-
-		if (options.panInside === false) {
-			return this.off('moveend', this._panInsideMaxBounds);
-		}
-
-		return this.on('moveend', this._panInsideMaxBounds);
 	},
 
 	setDocBounds: function (bounds) {
@@ -365,41 +355,12 @@ L.Map = L.Evented.extend({
 		return this.panTo(newCenter, options);
 	},
 
-	invalidateSize: function (options) {
+	invalidateSize: function () {
 		if (!this._loaded) { return this; }
 
-		options = L.extend({
-			animate: false,
-			pan: true
-		}, options === true ? {animate: true} : options);
-
 		var oldSize = this.getSize();
 		this._sizeChanged = true;
-
-		var newSize = this.getSize(),
-		    oldCenter = oldSize.divideBy(2).round(),
-		    newCenter = newSize.divideBy(2).round(),
-		    offset = oldCenter.subtract(newCenter);
-
-		if (!offset.x && !offset.y) { return this; }
-
-		if (options.animate && options.pan) {
-			this.panBy(offset);
-
-		} else {
-			if (options.pan) {
-				this._rawPanBy(offset);
-			}
-
-			this.fire('move');
-
-			if (options.debounceMoveend) {
-				clearTimeout(this._sizeTimer);
-				this._sizeTimer = setTimeout(L.bind(this.fire, this, 'moveend'), 200);
-			} else {
-				this.fire('moveend');
-			}
-		}
+		var newSize = this.getSize();
 
 		return this.fire('resize', {
 			oldSize: oldSize,
@@ -675,7 +636,7 @@ L.Map = L.Evented.extend({
 		console.debug('focus:');
 		if (this._docLayer) {
 			console.debug('focus: focussing');
-			this._clipboardContainer.focus(true);
+			this._clipboardContainer.focus();
 		}
 	},
 
@@ -809,10 +770,6 @@ L.Map = L.Evented.extend({
 		return this.getMaxZoom() - this.getMinZoom();
 	},
 
-	_panInsideMaxBounds: function () {
-		this.panInsideBounds(this.options.maxBounds);
-	},
-
 	_checkIfLoaded: function () {
 		if (!this._loaded) {
 			throw new Error('Set map center and zoom first.');
@@ -845,7 +802,7 @@ L.Map = L.Evented.extend({
 	_onResize: function () {
 		L.Util.cancelAnimFrame(this._resizeRequest);
 		this._resizeRequest = L.Util.requestAnimFrame(
-		        function () { this.invalidateSize({debounceMoveend: true}); }, this, false, this._container);
+		        function () { this.invalidateSize(); }, this, false, this._container);
 	},
 
 	_activate: function () {
@@ -875,7 +832,9 @@ L.Map = L.Evented.extend({
 
 					vex.dialogID = -1;
 					this._startInactiveTimer();
-					this.focus();
+					if (!L.Browser.mobile) {
+						this.focus();
+					}
 					return vex.close(id);
 				}
 			} else {
@@ -884,7 +843,9 @@ L.Map = L.Evented.extend({
 		}
 
 		this._startInactiveTimer();
-		this.focus();
+		if (!L.Browser.mobile) {
+			this.focus();
+		}
 		return false;
 	},
 
@@ -1080,8 +1041,7 @@ L.Map = L.Evented.extend({
 		// Calling from some other place with no real 'click' event doesn't work
 		if (type === 'click') {
 			if (this._permission === 'edit') {
-				this._clipboardContainer.focus(false);
-				this._clipboardContainer.focus(true);
+				this.focus();
 			}
 
 			// unselect if anything is selected already
