@@ -19,36 +19,40 @@
 #include "common/Unit.hpp"
 #include "Socket.hpp"
 
+#include <Poco/MemoryStream.h>
 #include <Poco/Net/HTTPRequest.h>
+#include <Poco/Net/HTTPResponse.h>
 #include <Poco/Net/WebSocket.h>
 
 class WebSocketHandler : public SocketHandlerInterface
 {
 protected:
-    // The socket that owns us (we can't own it).
+    /// The socket that owns us (we can't own it).
     std::weak_ptr<StreamSocket> _socket;
 
-    const int InitialPingDelayMs = 25;
-    const int PingFrequencyMs = 18 * 1000;
-    std::chrono::steady_clock::time_point _pingSent;
+    std::chrono::steady_clock::time_point _lastPingSentTime;
     int _pingTimeUs;
 
     std::vector<char> _wsPayload;
-    bool _shuttingDown;
-    enum class WSState { HTTP, WS } _wsState;
+    std::atomic<bool> _shuttingDown;
+    bool _isClient;
 
-    enum class WSFrameMask : unsigned char
+    struct WSFrameMask
     {
-        Fin = 0x80,
-        Mask = 0x80
+        static const unsigned char Fin = 0x80;
+        static const unsigned char Mask = 0x80;
     };
 
+    static const int InitialPingDelayMs;
+    static const int PingFrequencyMs;
+
 public:
-    WebSocketHandler() :
-        _pingSent(std::chrono::steady_clock::now()),
+    /// Perform upgrade ourselves, or select a client web socket.
+    WebSocketHandler(bool isClient = false) :
+        _lastPingSentTime(std::chrono::steady_clock::now()),
         _pingTimeUs(0),
         _shuttingDown(false),
-        _wsState(WSState::HTTP)
+        _isClient(isClient)
     {
     }
 
@@ -56,12 +60,12 @@ public:
     WebSocketHandler(const std::weak_ptr<StreamSocket>& socket,
                      const Poco::Net::HTTPRequest& request) :
         _socket(socket),
-        _pingSent(std::chrono::steady_clock::now() -
+        _lastPingSentTime(std::chrono::steady_clock::now() -
                   std::chrono::milliseconds(PingFrequencyMs) -
                   std::chrono::milliseconds(InitialPingDelayMs)),
         _pingTimeUs(0),
         _shuttingDown(false),
-        _wsState(WSState::HTTP)
+        _isClient(false)
     {
         upgradeToWebSocket(request);
     }
@@ -94,7 +98,7 @@ public:
     /// Sends WS shutdown message to the peer.
     void shutdown(const StatusCodes statusCode = StatusCodes::NORMAL_CLOSE, const std::string& statusMessage = "")
     {
-        auto socket = _socket.lock();
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
         if (socket == nullptr)
         {
             LOG_ERR("No socket associated with WebSocketHandler 0x" << std::hex << this << std::dec);
@@ -110,7 +114,7 @@ public:
         buf[0] = ((((int)statusCode) >> 8) & 0xff);
         buf[1] = ((((int)statusCode) >> 0) & 0xff);
         std::copy(statusMessage.begin(), statusMessage.end(), buf.begin() + 2);
-        const unsigned char flags = static_cast<unsigned char>(WSFrameMask::Fin)
+        const unsigned char flags = WSFrameMask::Fin
                                   | static_cast<char>(WSOpCode::Close);
 
         sendFrame(socket, buf.data(), buf.size(), flags);
@@ -126,7 +130,7 @@ public:
         if (len == 0)
             return false; // avoid logging.
 
-        LOG_TRC("#" << socket->getFD() << ": Incoming WebSocket data of " << len << " bytes.");
+        LOG_TRC("#" << socket->getFD() << ": Incoming WebSocket data of " << len << " bytes: " << std::string(socket->_inBuffer.data(), socket->_inBuffer.size()));
 
         if (len < 2) // partial read
             return false;
@@ -194,21 +198,51 @@ public:
                 ", fin? " << fin << ", mask? " << hasMask << ", payload length: " << _wsPayload.size() <<
                 ", residual socket data: " << socket->_inBuffer.size() << " bytes.");
 
+        bool doClose = false;
+
         switch (code)
         {
         case WSOpCode::Pong:
-            _pingTimeUs = std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::steady_clock::now() - _pingSent).count();
-            LOG_TRC("#" << socket->getFD() << ": Pong received: " << _pingTimeUs << " microseconds");
-            break;
+        {
+            if (_isClient)
+            {
+                LOG_ERR("#" << socket->getFD() << ": Servers should not send pongs, only clients");
+                doClose = true;
+                break;
+            }
+            else
+            {
+                _pingTimeUs = std::chrono::duration_cast<std::chrono::microseconds>
+                    (std::chrono::steady_clock::now() - _lastPingSentTime).count();
+                LOG_TRC("#" << socket->getFD() << ": Pong received: " << _pingTimeUs << " microseconds");
+                break;
+            }
+        }
         case WSOpCode::Ping:
-            LOG_ERR("#" << socket->getFD() << ": Clients should not send pings, only servers");
-            // drop through
-#if defined __clang__
-            [[clang::fallthrough]];
-#elif defined __GNUC__ && __GNUC__ >= 7
-            [[fallthrough]];
-#endif
+            if (_isClient)
+            {
+                auto now = std::chrono::steady_clock::now();
+                _pingTimeUs = std::chrono::duration_cast<std::chrono::microseconds>
+                                        (now - _lastPingSentTime).count();
+                sendPong(now, &_wsPayload[0], payloadLen, socket);
+                break;
+            }
+            else
+            {
+                LOG_ERR("#" << socket->getFD() << ": Clients should not send pings, only servers");
+                doClose = true;
+            }
+            break;
         case WSOpCode::Close:
+            doClose = true;
+            break;
+        default:
+            handleMessage(fin, code, _wsPayload);
+            break;
+        }
+
+        if (doClose)
+        {
             if (!_shuttingDown)
             {
                 // Peer-initiated shutdown must be echoed.
@@ -233,10 +267,6 @@ public:
 
             // TCP Close.
             socket->closeConnection();
-            break;
-        default:
-            handleMessage(fin, code, _wsPayload);
-            break;
         }
 
         _wsPayload.clear();
@@ -247,11 +277,13 @@ public:
     /// Implementation of the SocketHandlerInterface.
     virtual void handleIncomingMessage(SocketDisposition&) override
     {
-        auto socket = _socket.lock();
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
         if (socket == nullptr)
         {
             LOG_ERR("No socket associated with WebSocketHandler 0x" << std::hex << this << std::dec);
         }
+        else if (_isClient && !socket->isWebSocket())
+            handleClientUpgrade();
         else
         {
             while (handleOneIncomingMessage(socket))
@@ -262,40 +294,64 @@ public:
     int getPollEvents(std::chrono::steady_clock::time_point now,
                       int & timeoutMaxMs) override
     {
-        const int timeSincePingMs =
-            std::chrono::duration_cast<std::chrono::milliseconds>(now - _pingSent).count();
-        timeoutMaxMs = std::min(timeoutMaxMs, PingFrequencyMs - timeSincePingMs);
+        if (!_isClient)
+        {
+            const int timeSincePingMs =
+                std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastPingSentTime).count();
+            timeoutMaxMs = std::min(timeoutMaxMs, PingFrequencyMs - timeSincePingMs);
+        }
         return POLLIN;
     }
 
     /// Send a ping message
-    void sendPing(std::chrono::steady_clock::time_point now,
-                  const std::shared_ptr<Socket>& socket)
+    void sendPingOrPong(std::chrono::steady_clock::time_point now,
+                        const char* data, const size_t len,
+                        const WSOpCode code,
+                        const std::shared_ptr<StreamSocket>& socket)
     {
         assert(socket && "Expected a valid socket instance.");
 
         // Must not send this before we're upgraded.
-        if (_wsState != WSState::WS)
+        if (!socket->isWebSocket())
         {
-            LOG_WRN("Attempted ping on non-upgraded websocket!");
-            _pingSent = now; // Pretend we sent it to avoid timing out immediately.
+            LOG_WRN("Attempted ping on non-upgraded websocket! #" << socket->getFD());
+            _lastPingSentTime = now; // Pretend we sent it to avoid timing out immediately.
             return;
         }
 
-        LOG_TRC("#" << socket->getFD() << ": Sending ping.");
+        LOG_TRC("#" << socket->getFD() << ": Sending " <<
+                (const char *)(code == WSOpCode::Ping ? " ping." : "pong."));
         // FIXME: allow an empty payload.
-        sendMessage("", 1, WSOpCode::Ping, false);
-        _pingSent = now;
+        sendMessage(data, len, code, false);
+        _lastPingSentTime = now;
+    }
+
+    void sendPing(std::chrono::steady_clock::time_point now,
+                  const std::shared_ptr<StreamSocket>& socket)
+    {
+        assert(!_isClient);
+        sendPingOrPong(now, "", 1, WSOpCode::Ping, socket);
+    }
+
+    void sendPong(std::chrono::steady_clock::time_point now,
+                  const char* data, const size_t len,
+                  const std::shared_ptr<StreamSocket>& socket)
+    {
+        assert(_isClient);
+        sendPingOrPong(now, data, len, WSOpCode::Pong, socket);
     }
 
     /// Do we need to handle a timeout ?
     void checkTimeout(std::chrono::steady_clock::time_point now) override
     {
+        if (_isClient)
+            return;
+
         const int timeSincePingMs =
-            std::chrono::duration_cast<std::chrono::milliseconds>(now - _pingSent).count();
+            std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastPingSentTime).count();
         if (timeSincePingMs >= PingFrequencyMs)
         {
-            const std::shared_ptr<Socket> socket = _socket.lock();
+            const std::shared_ptr<StreamSocket> socket = _socket.lock();
             if (socket)
                 sendPing(now, socket);
         }
@@ -316,13 +372,13 @@ public:
     int sendMessage(const char* data, const size_t len, const WSOpCode code, const bool flush = true) const
     {
         int unitReturn = -1;
-        if (UnitWSD::get().filterSendMessage(data, len, code, flush, unitReturn))
+        if (UnitBase::get().filterSendMessage(data, len, code, flush, unitReturn))
             return unitReturn;
 
         //TODO: Support fragmented messages.
 
-        auto socket = _socket.lock();
-        return sendFrame(socket, data, len, static_cast<unsigned char>(WSFrameMask::Fin) | static_cast<unsigned char>(code), flush);
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
+        return sendFrame(socket, data, len, WSFrameMask::Fin | static_cast<unsigned char>(code), flush);
     }
 
 protected:
@@ -337,6 +393,9 @@ protected:
         if (!socket || data == nullptr || len == 0)
             return -1;
 
+        if (socket->isClosed())
+            return 0;
+
         socket->assertCorrectThread();
         std::vector<char>& out = socket->_outBuffer;
         const size_t oldSize = out.size();
@@ -398,12 +457,12 @@ protected:
     /// Upgrade the http(s) connection to a websocket.
     void upgradeToWebSocket(const Poco::Net::HTTPRequest& req)
     {
-        auto socket = _socket.lock();
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
         if (socket == nullptr)
             throw std::runtime_error("Invalid socket while upgrading to WebSocket. Request: " + req.getURI());
 
         LOG_TRC("#" << socket->getFD() << ": Upgrading to WebSocket.");
-        assert(_wsState == WSState::HTTP);
+        assert(!socket->isWebSocket());
 
         // create our websocket goodness ...
         const int wsVersion = std::stoi(req.get("Sec-WebSocket-Version", "13"));
@@ -428,12 +487,82 @@ protected:
         const std::string res = oss.str();
         LOG_TRC("#" << socket->getFD() << ": Sending WS Upgrade response: " << res);
         socket->send(res);
+        setWebSocket();
+    }
 
-        _wsState = WSState::WS;
+    // Handle incoming upgrade to full socket as client WS.
+    void handleClientUpgrade()
+    {
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
+
+        LOG_TRC("Incoming client websocket upgrade response: " << std::string(&socket->_inBuffer[0], socket->_inBuffer.size()));
+
+        bool bOk = false;
+        size_t responseSize = 0;
+
+        try
+        {
+            Poco::MemoryInputStream message(&socket->_inBuffer[0], socket->_inBuffer.size());;
+            Poco::Net::HTTPResponse response;
+
+            response.read(message);
+
+            {
+                static const std::string marker("\r\n\r\n");
+                auto itBody = std::search(socket->_inBuffer.begin(),
+                                          socket->_inBuffer.end(),
+                                          marker.begin(), marker.end());
+
+                if (itBody != socket->_inBuffer.end())
+                    responseSize = itBody - socket->_inBuffer.begin() + marker.size();
+            }
+
+            if (response.getStatus() == Poco::Net::HTTPResponse::HTTP_SWITCHING_PROTOCOLS &&
+                    response.has("Upgrade") && Poco::icompare(response.get("Upgrade"), "websocket") == 0)
+            {
+#if 0 // SAL_DEBUG ...
+                const std::string wsKey = response.get("Sec-WebSocket-Accept", "");
+                const std::string wsProtocol = response.get("Sec-WebSocket-Protocol", "");
+                if (Poco::icompare(wsProtocol, "chat") != 0)
+                    LOG_ERR("Unknown websocket protocol " << wsProtocol);
+                else
+#endif
+                {
+                    LOG_TRC("Accepted incoming websocket response");
+                    // FIXME: validate Sec-WebSocket-Accept vs. Sec-WebSocket-Key etc.
+                    bOk = true;
+                }
+            }
+        }
+        catch (const Poco::Exception& exc)
+        {
+            LOG_DBG("handleClientUpgrade exception caught: " << exc.displayText());
+        }
+        catch (const std::exception& exc)
+        {
+            LOG_DBG("handleClientUpgrade exception caught.");
+        }
+
+        if (!bOk || responseSize == 0)
+        {
+            LOG_ERR("Bad websocker server response.");
+
+            socket->shutdown();
+            return;
+        }
+
+        setWebSocket();
+        socket->eraseFirstInputBytes(responseSize);
+    }
+
+    void setWebSocket()
+    {
+        std::shared_ptr<StreamSocket> socket = _socket.lock();
+        socket->setWebSocket();
 
         // No need to ping right upon connection/upgrade,
         // but do reset the time to avoid pinging immediately after.
-        _pingSent = std::chrono::steady_clock::now();
+        _lastPingSentTime = std::chrono::steady_clock::now();
     }
 };
 
