@@ -7,7 +7,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "DocumentBroker.hpp"
 
@@ -33,10 +33,13 @@
 #include "SenderQueue.hpp"
 #include "Storage.hpp"
 #include "TileCache.hpp"
-#include "common/Log.hpp"
-#include "common/Message.hpp"
-#include "common/Protocol.hpp"
-#include "common/Unit.hpp"
+#include <common/Log.hpp>
+#include <common/Message.hpp>
+#include <common/Protocol.hpp>
+#include <common/Unit.hpp>
+
+#include <sys/types.h>
+#include <sys/wait.h>
 
 using namespace LOOLProtocol;
 
@@ -71,7 +74,7 @@ Poco::URI DocumentBroker::sanitizeURI(const std::string& uri)
     // The URI of the document should be url-encoded.
     std::string decodedUri;
     Poco::URI::decode(uri, decodedUri);
-    auto uriPublic = Poco::URI(decodedUri);
+    Poco::URI uriPublic(decodedUri);
 
     if (uriPublic.isRelative() || uriPublic.getScheme() == "file")
     {
@@ -152,7 +155,7 @@ DocumentBroker::DocumentBroker(const std::string& uri,
     _lastSaveTime(std::chrono::steady_clock::now()),
     _lastSaveRequestTime(std::chrono::steady_clock::now() - std::chrono::milliseconds(COMMAND_TIMEOUT_MS)),
     _markToDestroy(false),
-    _lastEditableSession(false),
+    _closeRequest(false),
     _isLoaded(false),
     _isModified(false),
     _cursorPosX(0),
@@ -177,7 +180,7 @@ void DocumentBroker::startThread()
     _poll->startThread();
 }
 
-void DocumentBroker::assertCorrectThread()
+void DocumentBroker::assertCorrectThread() const
 {
     _poll->assertCorrectThread();
 }
@@ -199,7 +202,7 @@ void DocumentBroker::pollThread()
                                                                   _threadStart).count() > timeoutMs)
             break;
 
-        // Nominal time between retries, lest we  busy-loop. getNewChild could also wait, so don't double that here.
+        // Nominal time between retries, lest we busy-loop. getNewChild could also wait, so don't double that here.
         std::this_thread::sleep_for(std::chrono::milliseconds(CHILD_REBALANCE_INTERVAL_MS / 10));
     }
     while (!_stop && _poll->continuePolling() && !TerminationFlag && !ShutdownRequestFlag);
@@ -210,14 +213,21 @@ void DocumentBroker::pollThread()
         LOG_ERR("Failed to get new child.");
 
         // FIXME: need to notify all clients and shut this down ...
+        // FIXME: return something good down the websocket ...
 #if 0
         const std::string msg = SERVICE_UNAVAILABLE_INTERNAL_ERROR;
         ws.sendMessage(msg);
         // abnormal close frame handshake
         ws.shutdown(WebSocketHandler::StatusCodes::ENDPOINT_GOING_AWAY);
 #endif
-        // FIXME: return something good down the websocket ...
-        _stop = true;
+        stop("Failed to get new child.");
+
+        // Stop to mark it done and cleanup.
+        _poll->stop();
+        _poll->removeSockets();
+
+        // Async cleanup.
+        LOOLWSD::doHousekeeping();
 
         LOG_INF("Finished docBroker polling thread for docKey [" << _docKey << "].");
         return;
@@ -226,15 +236,15 @@ void DocumentBroker::pollThread()
     _childProcess->setDocumentBroker(shared_from_this());
     LOG_INF("Doc [" << _docKey << "] attached to child [" << _childProcess->getPid() << "].");
 
-    auto last30SecCheckTime = std::chrono::steady_clock::now();
-
     static const bool AutoSaveEnabled = !std::getenv("LOOL_NO_AUTOSAVE");
     static const size_t IdleDocTimeoutSecs = LOOLWSD::getConfigValue<int>(
                                                       "per_document.idle_timeout_secs", 3600);
+
     // Used to accumulate B/W deltas.
     uint64_t adminSent = 0;
     uint64_t adminRecv = 0;
     auto lastBWUpdateTime = std::chrono::steady_clock::now();
+    auto last30SecCheckTime = std::chrono::steady_clock::now();
 
     // Main polling loop goodness.
     while (!_stop && _poll->continuePolling() && !TerminationFlag)
@@ -254,23 +264,28 @@ void DocumentBroker::pollThread()
                                        // connection drop transiently reduces this.
                                        (sent > adminSent ? (sent - adminSent): uint64_t(0)),
                                        (recv > adminRecv ? (recv - adminRecv): uint64_t(0)));
-            LOG_INF("Doc [" << _docKey << "] added sent: " << sent << " recv: " << recv << " bytes to totals");
+            LOG_DBG("Doc [" << _docKey << "] added sent: " << sent << " recv: " << recv << " bytes to totals");
             adminSent = sent;
             adminRecv = recv;
         }
 
-        if (_lastSaveTime < _lastSaveRequestTime &&
+        if (isSaving() &&
             std::chrono::duration_cast<std::chrono::milliseconds>
                     (now - _lastSaveRequestTime).count() <= COMMAND_TIMEOUT_MS)
         {
-            // We are saving, nothing more to do but wait.
+            // We are saving, nothing more to do but wait (until we save or we timeout).
             continue;
         }
 
-        if (ShutdownRequestFlag)
+        if (ShutdownRequestFlag || _closeRequest)
         {
-            _closeReason = "recycling";
-            _stop = true;
+            const std::string reason = ShutdownRequestFlag ? "recycling" : _closeReason;
+            LOG_INF("Autosaving DocumentBroker for docKey [" << getDocKey() << "] for " << reason);
+            if (!autoSave(isPossiblyModified()))
+            {
+                LOG_INF("Terminating DocumentBroker for docKey [" << getDocKey() << "].");
+                stop(reason);
+            }
         }
         else if (AutoSaveEnabled && !_stop &&
                  std::chrono::duration_cast<std::chrono::seconds>(now - last30SecCheckTime).count() >= 30)
@@ -281,21 +296,35 @@ void DocumentBroker::pollThread()
         }
 
         // Remove idle documents after 1 hour.
-        const bool idle = (getIdleTimeSecs() >= IdleDocTimeoutSecs);
-
-        // If all sessions have been removed, no reason to linger.
-        if ((isLoaded() || _markToDestroy) && (_sessions.empty() || idle))
+        const bool idle = (isLoaded() && getIdleTimeSecs() >= IdleDocTimeoutSecs);
+        if (idle)
         {
-            LOG_INF("Terminating " << (idle ? "idle" : "dead") <<
-                    " DocumentBroker for docKey [" << getDocKey() << "].");
-            _closeReason = (idle ? "idle" : "dead");
-            _stop = true;
+            // Stop if there is nothing to save.
+            LOG_INF("Autosaving idle DocumentBroker for docKey [" << getDocKey() << "] to kill.");
+            if (!autoSave(isPossiblyModified()))
+            {
+                LOG_INF("Terminating idle DocumentBroker for docKey [" << getDocKey() << "].");
+                stop("idle");
+            }
+        }
+        else if (_sessions.empty() && (isLoaded() || _markToDestroy))
+        {
+            // If all sessions have been removed, no reason to linger.
+            LOG_INF("Terminating dead DocumentBroker for docKey [" << getDocKey() << "].");
+            stop("dead");
         }
     }
 
     LOG_INF("Finished polling doc [" << _docKey << "]. stop: " << _stop << ", continuePolling: " <<
             _poll->continuePolling() << ", ShutdownRequestFlag: " << ShutdownRequestFlag <<
-            ", TerminationFlag: " << TerminationFlag << ".");
+            ", TerminationFlag: " << TerminationFlag << ", closeReason: " << _closeReason << ". Flushing socket.");
+
+    if (_isModified)
+    {
+        std::stringstream state;
+        dumpState(state);
+        LOG_ERR("DocumentBroker stopping although modified " << state.str());
+    }
 
     // Flush socket data first.
     const int flushTimeoutMs = POLL_TIMEOUT_MS * 2; // ~1000ms
@@ -310,6 +339,10 @@ void DocumentBroker::pollThread()
         _poll->poll(std::min(flushTimeoutMs - elapsedMs, POLL_TIMEOUT_MS / 5));
     }
 
+    LOG_INF("Finished flushing socket for doc [" << _docKey << "]. stop: " << _stop << ", continuePolling: " <<
+            _poll->continuePolling() << ", ShutdownRequestFlag: " << ShutdownRequestFlag <<
+            ", TerminationFlag: " << TerminationFlag << ". Terminating child with reason: [" << _closeReason << "].");
+
     // Terminate properly while we can.
     terminateChild(_closeReason);
 
@@ -363,8 +396,10 @@ void DocumentBroker::joinThread()
     _poll->joinThread();
 }
 
-void DocumentBroker::stop()
+void DocumentBroker::stop(const std::string& reason)
 {
+    LOG_DBG("Closing DocumentBroker for docKey [" << _docKey << "] with reason: " << reason);
+    _closeReason = reason; // used later in the polling loop
     _stop = true;
     _poll->wakeup();
 }
@@ -396,7 +431,7 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
     // We need to map it to a jailed path and copy the file there.
 
     // user/doc/jailId
-    const auto jailPath = Poco::Path(JAILED_DOCUMENT_ROOT, jailId);
+    const Poco::Path jailPath(JAILED_DOCUMENT_ROOT, jailId);
     std::string jailRoot = getJailRoot();
 
     LOG_INF("jailPath: " << jailPath.toString() << ", jailRoot: " << jailRoot);
@@ -416,7 +451,6 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
             LOG_ERR("Failed to create Storage instance for [" << _docKey << "] in " << jailPath.toString());
             return false;
         }
-
         firstInstance = true;
     }
 
@@ -468,6 +502,7 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
         wopiInfo->set("HidePrintOption", wopifileinfo->_hidePrintOption);
         wopiInfo->set("HideSaveOption", wopifileinfo->_hideSaveOption);
         wopiInfo->set("HideExportOption", wopifileinfo->_hideExportOption);
+        wopiInfo->set("HideChangeTrackingControls", wopifileinfo->_hideChangeTrackingControls);
         wopiInfo->set("DisablePrint", wopifileinfo->_disablePrint);
         wopiInfo->set("DisableExport", wopifileinfo->_disableExport);
         wopiInfo->set("DisableCopy", wopifileinfo->_disableCopy);
@@ -515,14 +550,15 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
         watermarkText = LOOLWSD::OverrideWatermark;
 #endif
 
-    LOG_DBG("Setting username [" << username << "] and userId [" << userid << "] for session [" << sessionId << "]");
+    LOG_DBG("Setting username [" << username << "] and userId [" <<
+            userid << "] for session [" << sessionId << "]");
     session->setUserId(userid);
     session->setUserName(username);
     session->setUserExtraInfo(userExtraInfo);
     session->setWatermarkText(watermarkText);
 
     // Basic file information was stored by the above getWOPIFileInfo() or getLocalFileInfo() calls
-    const auto fileInfo = _storage->getFileInfo();
+    const StorageBase::FileInfo fileInfo = _storage->getFileInfo();
     if (!fileInfo.isValid())
     {
         LOG_ERR("Invalid fileinfo for URI [" << session->getPublicUri().toString() << "].");
@@ -532,20 +568,20 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
     if (firstInstance)
     {
         _documentLastModifiedTime = fileInfo._modifiedTime;
-        Log::debug() << "Document timestamp: " << _documentLastModifiedTime << Log::end;
+        LOG_DBG("Document timestamp: " << _documentLastModifiedTime);
     }
     else
     {
         // Check if document has been modified by some external action
-        Log::trace() << "Document modified time: " << fileInfo._modifiedTime << Log::end;
+        LOG_TRC("Document modified time: " << fileInfo._modifiedTime);
         static const Poco::Timestamp Zero(Poco::Timestamp::fromEpochTime(0));
         if (_documentLastModifiedTime != Zero &&
             fileInfo._modifiedTime != Zero &&
             _documentLastModifiedTime != fileInfo._modifiedTime)
         {
-            Log::trace() << "Document " << _docKey << "] has been modified behind our back. Informing all clients."
-                         << "Expected: " << _documentLastModifiedTime
-                         << "Actual: " << fileInfo._modifiedTime << Log::end;
+            LOG_TRC("Document " << _docKey << "] has been modified behind our back. " <<
+                    "Informing all clients. Expected: " << _documentLastModifiedTime <<
+                    ", Actual: " << fileInfo._modifiedTime);
 
             _documentChangedInStorage = true;
             std::string message = "close: documentconflict";
@@ -560,16 +596,16 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
     // Let's load the document now, if not loaded.
     if (!_storage->isLoaded())
     {
-        auto localPath = _storage->loadStorageFileToLocal(session->getAuthorization());
+        std::string localPath = _storage->loadStorageFileToLocal(session->getAuthorization());
 
         // Check if we have a prefilter "plugin" for this document format
         for (const auto& plugin : LOOLWSD::PluginConfigurations)
         {
             try
             {
-                std::string extension(plugin->getString("prefilter.extension"));
-                std::string newExtension(plugin->getString("prefilter.newextension"));
-                std::string commandLine(plugin->getString("prefilter.commandline"));
+                const std::string extension(plugin->getString("prefilter.extension"));
+                const std::string newExtension(plugin->getString("prefilter.newextension"));
+                const std::string commandLine(plugin->getString("prefilter.commandline"));
 
                 if (localPath.length() > extension.length()+1 &&
                     strcasecmp(localPath.substr(localPath.length() - extension.length() -1).data(), (std::string(".") + extension).data()) == 0)
@@ -577,28 +613,29 @@ bool DocumentBroker::load(const std::shared_ptr<ClientSession>& session, const s
                     // Extension matches, try the conversion. We convert the file to another one in
                     // the same (jail) directory, with just the new extension tacked on.
 
-                    std::string newRootPath = _storage->getRootFilePath() + "." + newExtension;
+                    const std::string newRootPath = _storage->getRootFilePath() + "." + newExtension;
 
                     // The commandline must contain the space-separated substring @INPUT@ that is
                     // replaced with the input file name, and @OUTPUT@ for the output file name.
-
-                    std::vector<std::string>args;
-
                     Poco::StringTokenizer tokenizer(commandLine, " ");
                     if (tokenizer.replace("@INPUT@", _storage->getRootFilePath()) != 1 ||
                         tokenizer.replace("@OUTPUT@", newRootPath) != 1)
                         throw Poco::NotFoundException();
 
-                    for (std::size_t i = 1; i < tokenizer.count(); i++)
-                        args.push_back(tokenizer[i]);
 
-                    Poco::ProcessHandle process = Poco::Process::launch(tokenizer[0], args);
-                    int rc = process.wait();
+                    std::vector<std::string> args;
+                    for (std::size_t i = 1; i < tokenizer.count(); ++i)
+                        args.emplace_back(tokenizer[i]);
+
+                    int process = Util::spawnProcess(tokenizer[0], args);
+                    int status = -1;
+                    const int rc = ::waitpid(process, &status, 0);
                     if (rc != 0)
                     {
-                        LOG_ERR("Conversion from " << extension << " to " << newExtension <<" failed");
+                        LOG_ERR("Conversion from " << extension << " to " << newExtension << " failed (" << rc << ").");
                         return false;
                     }
+
                     _storage->setRootFilePath(newRootPath);
                     localPath += "." + newExtension;
                 }
@@ -661,6 +698,7 @@ bool DocumentBroker::saveToStorage(const std::string& sessionId,
         LOG_TRC("Document will be saved forcefully to storage.");
         _storage->forceSave();
     }
+
     const bool res = saveToStorageInternal(sessionId, success, result);
 
     // If marked to destroy, or session is disconnected, remove.
@@ -691,6 +729,9 @@ bool DocumentBroker::saveToStorageInternal(const std::string& sessionId,
 {
     assertCorrectThread();
 
+    // Record that we got a response to avoid timeing out on saving.
+    _lastSaveResponseTime = std::chrono::steady_clock::now();
+
     const bool isSaveAs = !saveAsPath.empty();
 
     // If save requested, but core didn't save because document was unmodified
@@ -721,17 +762,15 @@ bool DocumentBroker::saveToStorageInternal(const std::string& sessionId,
     }
 
     const Authorization auth = it->second->getAuthorization();
-    const auto uri = isSaveAs? saveAsPath: it->second->getPublicUri().toString();
+    const std::string uri = isSaveAs? saveAsPath: it->second->getPublicUri().toString();
 
-    // If we aren't destroying the last editable session just yet,
-    // and the file timestamp hasn't changed, skip saving.
-    const auto newFileModifiedTime = Poco::File(_storage->getRootFilePath()).getLastModified();
-    if (!isSaveAs && !_lastEditableSession && newFileModifiedTime == _lastFileModifiedTime)
+    // If the file timestamp hasn't changed, skip saving.
+    const Poco::Timestamp newFileModifiedTime = Poco::File(_storage->getRootFilePath()).getLastModified();
+    if (!isSaveAs && newFileModifiedTime == _lastFileModifiedTime)
     {
         // Nothing to do.
         LOG_DBG("Skipping unnecessary saving to URI [" << uri << "] with docKey [" << _docKey <<
                 "]. File last modified " << _lastFileModifiedTime.elapsed() / 1000000 << " seconds ago.");
-        _lastSaveTime = std::chrono::steady_clock::now();
         _poll->wakeup();
         return true;
     }
@@ -744,21 +783,23 @@ bool DocumentBroker::saveToStorageInternal(const std::string& sessionId,
     {
         if (!isSaveAs)
         {
+            // Saved and stored; update flags.
             setModified(false);
             _lastFileModifiedTime = newFileModifiedTime;
             _tileCache->saveLastModified(_lastFileModifiedTime);
             _lastSaveTime = std::chrono::steady_clock::now();
-            _poll->wakeup();
 
-            // So set _documentLastModifiedTime then
+            // Save the storage timestamp.
             _documentLastModifiedTime = _storage->getFileInfo()._modifiedTime;
 
             // After a successful save, we are sure that document in the storage is same as ours
             _documentChangedInStorage = false;
 
-            Log::debug() << "Saved docKey [" << _docKey << "] to URI [" << uri
-                         << "] and updated tile cache. Document modified timestamp: "
-                         << _documentLastModifiedTime << Log::end;
+            LOG_DBG("Saved docKey [" << _docKey << "] to URI [" << uri << "] and updated timestamps. " <<
+                    " Document modified timestamp: " << _documentLastModifiedTime);
+
+            // Resume polling.
+            _poll->wakeup();
         }
         else
         {
@@ -771,8 +812,8 @@ bool DocumentBroker::saveToStorageInternal(const std::string& sessionId,
 
             it->second->sendTextFrame("saveas: url=" + url + " filename=" + encodedName);
 
-            Log::debug() << "Saved As docKey [" << _docKey << "] to URI [" << url
-                         << " with name '" << encodedName << "'] successfully.";
+            LOG_DBG("Saved As docKey [" << _docKey << "] to URI [" << url <<
+                    " with name '" << encodedName << "'] successfully.");
         }
         return true;
     }
@@ -803,14 +844,11 @@ bool DocumentBroker::saveToStorageInternal(const std::string& sessionId,
     {
         LOG_ERR("PutFile says that Document changed in storage");
         _documentChangedInStorage = true;
+        std::string message = "close: documentconflict";
         if (_isModified)
-        {
-            broadcastMessage("error: cmd=storage kind=documentconflict");
-        }
-        else
-        {
-            closeDocument("documentconflict");
-        }
+            message = "error: cmd=storage kind=documentconflict";
+
+        broadcastMessage(message);
     }
 
     return false;
@@ -831,6 +869,7 @@ bool DocumentBroker::autoSave(const bool force)
 {
     assertCorrectThread();
 
+    LOG_TRC("autoSave(): forceful? " << force);
     if (_sessions.empty() || _storage == nullptr || !_isLoaded ||
         !_childProcess->isAlive() || (!_isModified && !force))
     {
@@ -846,8 +885,8 @@ bool DocumentBroker::autoSave(const bool force)
     std::string savingSessionId;
     for (auto& sessionIt : _sessions)
     {
-        // Save the document using first session available ...
-        if (savingSessionId.empty())
+        // Save the document using an editable session, or first ...
+        if (savingSessionId.empty() || !sessionIt.second->isReadOnly())
         {
             savingSessionId = sessionIt.second->getId();
         }
@@ -864,18 +903,19 @@ bool DocumentBroker::autoSave(const bool force)
     if (force)
     {
         LOG_TRC("Sending forced save command for [" << _docKey << "].");
-        sent = sendUnoSave(savingSessionId);
+        // Don't terminate editing as this can be invoked by the admin OOM, but otherwise force saving anyway.
+        sent = sendUnoSave(savingSessionId, /*dontTerminateEdit=*/ true, /*dontSaveIfUnmodified=*/ true, /*isAutosave=*/ false);
     }
     else if (_isModified)
     {
-        const auto now = std::chrono::steady_clock::now();
-        const auto inactivityTimeMs = std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastActivityTime).count();
-        const auto timeSinceLastSaveMs = std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastSaveTime).count();
+        const std::chrono::steady_clock::time_point now = std::chrono::steady_clock::now();
+        const std::chrono::milliseconds::rep inactivityTimeMs = std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastActivityTime).count();
+        const std::chrono::milliseconds::rep timeSinceLastSaveMs = std::chrono::duration_cast<std::chrono::milliseconds>(now - _lastSaveTime).count();
         LOG_TRC("Time since last save of docKey [" << _docKey << "] is " << timeSinceLastSaveMs <<
                 "ms and most recent activity was " << inactivityTimeMs << "ms ago.");
 
-        static const auto idleSaveDurationMs = LOOLWSD::getConfigValue<int>("per_document.idlesave_duration_secs", 30) * 1000;
-        static const auto autoSaveDurationMs = LOOLWSD::getConfigValue<int>("per_document.autosave_duration_secs", 300) * 1000;
+        static const int idleSaveDurationMs = LOOLWSD::getConfigValue<int>("per_document.idlesave_duration_secs", 30) * 1000;
+        static const int autoSaveDurationMs = LOOLWSD::getConfigValue<int>("per_document.autosave_duration_secs", 300) * 1000;
         // Either we've been idle long enough, or it's auto-save time.
         if (inactivityTimeMs >= idleSaveDurationMs ||
             timeSinceLastSaveMs >= autoSaveDurationMs)
@@ -930,10 +970,9 @@ bool DocumentBroker::sendUnoSave(const std::string& sessionId, bool dontTerminat
         oss << "}";
 
         assert(_storage);
-        _storage->setUserModified(_isModified);
         _storage->setIsAutosave(isAutosave || UnitWSD::get().isAutosave());
 
-        const auto saveArgs = oss.str();
+        const std::string saveArgs = oss.str();
         LOG_TRC(".uno:Save arguments: " << saveArgs);
         const auto command = "uno .uno:Save " + saveArgs;
         forwardToChild(sessionId, command);
@@ -996,13 +1035,7 @@ size_t DocumentBroker::addSessionInternal(const std::shared_ptr<ClientSession>&
         throw;
     }
 
-    // Below values are recalculated when destroyIfLastEditor() is called (before destroying the
-    // document). It is safe to reset their values to their defaults whenever a new session is added.
-    _lastEditableSession = false;
-    _markToDestroy = false;
-    _stop = false;
-
-    const auto id = session->getId();
+    const std::string id = session->getId();
 
     // Request a new session from the child kit.
     const std::string aMessage = "session " + id + ' ' + _docKey + ' ' + _docId;
@@ -1015,7 +1048,7 @@ size_t DocumentBroker::addSessionInternal(const std::shared_ptr<ClientSession>&
     _sessions.emplace(session->getId(), session);
     session->setAttached();
 
-    const auto count = _sessions.size();
+    const size_t count = _sessions.size();
     LOG_TRC("Added " << (session->isReadOnly() ? "readonly" : "non-readonly") <<
             " session [" << id << "] to docKey [" <<
             _docKey << "] to have " << count << " sessions.");
@@ -1023,21 +1056,31 @@ size_t DocumentBroker::addSessionInternal(const std::shared_ptr<ClientSession>&
     return count;
 }
 
-size_t DocumentBroker::removeSession(const std::string& id, bool destroyIfLast)
+size_t DocumentBroker::removeSession(const std::string& id)
 {
     assertCorrectThread();
 
-    if (destroyIfLast)
-        destroyIfLastEditor(id);
-
     try
     {
+        const auto it = _sessions.find(id);
+        if (it == _sessions.end())
+        {
+            LOG_ERR("Invalid or unknown session [" << id << "] to remove.");
+            return _sessions.size();
+        }
+
+        // Last view going away, can destroy.
+        _markToDestroy = (_sessions.size() <= 1);
+
+        const bool lastEditableSession = !it->second->isReadOnly() && !haveAnotherEditableSession(id);
+
         LOG_INF("Removing session [" << id << "] on docKey [" << _docKey <<
                 "]. Have " << _sessions.size() << " sessions. markToDestroy: " << _markToDestroy <<
-                ", LastEditableSession: " << _lastEditableSession);
+                ", LastEditableSession: " << lastEditableSession);
 
-        if (!_lastEditableSession || !autoSave(true))
-            return removeSessionInternal(id);
+        // If last editable, save and don't remove until after uploading to storage.
+        if (!lastEditableSession || !autoSave(isPossiblyModified()))
+            removeSessionInternal(id);
     }
     catch (const std::exception& ex)
     {
@@ -1059,19 +1102,23 @@ size_t DocumentBroker::removeSessionInternal(const std::string& id)
         {
             LOOLWSD::dumpEndSessionTrace(getJailId(), id, _uriOrig);
 
-            const auto readonly = (it->second ? it->second->isReadOnly() : false);
+            const bool readonly = (it->second ? it->second->isReadOnly() : false);
 
             // Remove. The caller must have a reference to the session
             // in question, lest we destroy from underneith them.
             _sessions.erase(it);
+            const size_t count = _sessions.size();
 
-            const auto count = _sessions.size();
-            LOG_TRC("Removed " << (readonly ? "readonly" : "non-readonly") <<
-                    " session [" << id << "] from docKey [" <<
-                    _docKey << "] to have " << count << " sessions.");
-            for (const auto& pair : _sessions)
+            Log::StreamLogger logger = Log::trace();
+            if (logger.enabled())
             {
-                LOG_TRC("Session: " << pair.second->getName());
+                logger << "Removed " << (readonly ? "readonly" : "non-readonly")
+                       << " session [" << id << "] from docKey ["
+                       << _docKey << "] to have " << count << " sessions:";
+                for (const auto& pair : _sessions)
+                    logger << pair.second->getId() << ' ';
+
+                LOG_END(logger);
             }
 
             // Let the child know the client has disconnected.
@@ -1185,7 +1232,7 @@ void DocumentBroker::handleTileRequest(TileDesc& tile,
     std::unique_lock<std::mutex> lock(_mutex);
 
     tile.setVersion(++_tileVersion);
-    const auto tileMsg = tile.serialize();
+    const std::string tileMsg = tile.serialize();
     LOG_TRC("Tile request for " << tileMsg);
 
     std::unique_ptr<std::fstream> cachedTile = _tileCache->lookupTile(tile);
@@ -1204,7 +1251,7 @@ void DocumentBroker::handleTileRequest(TileDesc& tile,
 
         assert(cachedTile->is_open());
         cachedTile->seekg(0, std::ios_base::end);
-        const auto pos = output.size();
+        const size_t pos = output.size();
         std::streamsize size = cachedTile->tellg();
         output.resize(pos + size);
         cachedTile->seekg(0, std::ios_base::beg);
@@ -1263,7 +1310,7 @@ void DocumentBroker::handleTileCombinedRequest(TileCombined& tileCombined,
 
             assert(cachedTile->is_open());
             cachedTile->seekg(0, std::ios_base::end);
-            const auto pos = output.size();
+            const size_t pos = output.size();
             std::streamsize size = cachedTile->tellg();
             output.resize(pos + size);
             cachedTile->seekg(0, std::ios_base::beg);
@@ -1284,10 +1331,10 @@ void DocumentBroker::handleTileCombinedRequest(TileCombined& tileCombined,
 
     if (!tiles.empty())
     {
-        auto newTileCombined = TileCombined::create(tiles);
+        TileCombined newTileCombined = TileCombined::create(tiles);
 
         // Forward to child to render.
-        const auto req = newTileCombined.serialize("tilecombine");
+        const std::string req = newTileCombined.serialize("tilecombine");
         LOG_DBG("Sending residual tilecombine: " << req);
         _childProcess->sendTextFrame(req);
     }
@@ -1297,7 +1344,7 @@ void DocumentBroker::cancelTileRequests(const std::shared_ptr<ClientSession>& se
 {
     std::unique_lock<std::mutex> lock(_mutex);
 
-    const auto canceltiles = tileCache().cancelTiles(session);
+    const std::string canceltiles = tileCache().cancelTiles(session);
     if (!canceltiles.empty())
     {
         LOG_DBG("Forwarding canceltiles request: " << canceltiles);
@@ -1312,12 +1359,12 @@ void DocumentBroker::handleTileResponse(const std::vector<char>& payload)
 
     try
     {
-        const auto length = payload.size();
+        const size_t length = payload.size();
         if (firstLine.size() < static_cast<std::string::size_type>(length) - 1)
         {
-            const auto tile = TileDesc::parse(firstLine);
-            const auto buffer = payload.data();
-            const auto offset = firstLine.size() + 1;
+            const TileDesc tile = TileDesc::parse(firstLine);
+            const char* buffer = payload.data();
+            const size_t offset = firstLine.size() + 1;
 
             std::unique_lock<std::mutex> lock(_mutex);
 
@@ -1342,12 +1389,12 @@ void DocumentBroker::handleTileCombinedResponse(const std::vector<char>& payload
 
     try
     {
-        const auto length = payload.size();
+        const size_t length = payload.size();
         if (firstLine.size() < static_cast<std::string::size_type>(length) - 1)
         {
-            const auto tileCombined = TileCombined::parse(firstLine);
-            const auto buffer = payload.data();
-            auto offset = firstLine.size() + 1;
+            const TileCombined tileCombined = TileCombined::parse(firstLine);
+            const char* buffer = payload.data();
+            size_t offset = firstLine.size() + 1;
 
             std::unique_lock<std::mutex> lock(_mutex);
 
@@ -1369,50 +1416,34 @@ void DocumentBroker::handleTileCombinedResponse(const std::vector<char>& payload
     }
 }
 
-void DocumentBroker::destroyIfLastEditor(const std::string& id)
+bool DocumentBroker::haveAnotherEditableSession(const std::string& id) const
 {
     assertCorrectThread();
 
-    const auto currentSession = _sessions.find(id);
-    if (currentSession == _sessions.end())
+    for (const auto& it : _sessions)
     {
-        // We could be called before adding any sessions.
-        // For example when a socket disconnects before loading.
-        return;
-    }
-
-    // Check if the session being destroyed is the last non-readonly session or not.
-    _lastEditableSession = !currentSession->second->isReadOnly();
-    if (_lastEditableSession && !_sessions.empty())
-    {
-        for (const auto& it : _sessions)
+        if (it.second->getId() != id &&
+            it.second->isViewLoaded() &&
+            !it.second->isReadOnly())
         {
-            if (it.second->getId() != id &&
-                it.second->isViewLoaded() &&
-                !it.second->isReadOnly())
-            {
-                // Found another editable.
-                _lastEditableSession = false;
-                break;
-            }
+            // This is a loaded session that is non-readonly.
+            return true;
         }
     }
 
-    // Last view going away, can destroy.
-    _markToDestroy = (_sessions.size() <= 1);
-    LOG_DBG("startDestroy on session [" << id << "] on docKey [" << _docKey <<
-            "], sessions: " << _sessions.size() << " markToDestroy: " << _markToDestroy <<
-            ", lastEditableSession: " << _lastEditableSession);
+    // None found.
+    return false;
 }
 
 void DocumentBroker::setModified(const bool value)
 {
-    if(_isModified != value)
+    if (_isModified != value)
     {
         _isModified = value;
         Admin::instance().modificationAlert(_docKey, getPid(), value);
     }
 
+    _storage->setUserModified(value);
     _tileCache->setUnsavedChanges(value);
 }
 
@@ -1520,7 +1551,7 @@ void DocumentBroker::shutdownClients(const std::string& closeReason)
             session->shutdown(WebSocketHandler::StatusCodes::ENDPOINT_GOING_AWAY, closeReason);
 
             // Remove session, save, and mark to destroy.
-            removeSession(session->getId(), true);
+            removeSession(session->getId());
         }
         catch (const std::exception& exc)
         {
@@ -1550,20 +1581,17 @@ void DocumentBroker::terminateChild(const std::string& closeReason)
 
     LOG_INF("Terminating doc [" << _docKey << "] with reason: " << closeReason);
 
-    // Close all running sessions
+    // Close all running sessions first.
     shutdownClients(closeReason);
 
     if (_childProcess)
     {
         LOG_INF("Terminating child [" << getPid() << "] of doc [" << _docKey << "].");
 
-        // First flag to stop as it might be waiting on our lock
-        // to process some incoming message.
-        _childProcess->stop();
-        _childProcess->close(false);
+        _childProcess->close();
     }
 
-    _stop = true;
+    stop(closeReason);
 }
 
 void DocumentBroker::closeDocument(const std::string& reason)
@@ -1571,8 +1599,8 @@ void DocumentBroker::closeDocument(const std::string& reason)
     assertCorrectThread();
 
     LOG_DBG("Closing DocumentBroker for docKey [" << _docKey << "] with reason: " << reason);
-    _closeReason = reason; // used later in the polling loop
-    stop();
+    _closeReason = reason;
+    _closeRequest = true;
 }
 
 void DocumentBroker::broadcastMessage(const std::string& message)
@@ -1632,8 +1660,7 @@ void DocumentBroker::dumpState(std::ostream& os)
     os << "\n  doc key: " << _docKey;
     os << "\n  doc id: " << _docId;
     os << "\n  num sessions: " << _sessions.size();
-    os << "\n  last editable?: " << _lastEditableSession;
-    std::time_t t = std::chrono::system_clock::to_time_t(
+    const std::time_t t = std::chrono::system_clock::to_time_t(
         std::chrono::system_clock::now()
         + (_lastSaveTime - std::chrono::steady_clock::now()));
     os << "\n  last saved: " << std::ctime(&t);
