@@ -7,7 +7,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <iomanip>
 #include <string>
@@ -17,6 +17,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <zlib.h>
+#include <security/pam_appl.h>
 
 #include <openssl/evp.h>
 
@@ -39,11 +40,12 @@
 #include <Poco/URI.h>
 
 #include "Auth.hpp"
-#include "Common.hpp"
+#include <Common.hpp>
+#include <Crypto.hpp>
 #include "FileServer.hpp"
 #include "LOOLWSD.hpp"
-#include "Log.hpp"
-#include "Protocol.hpp"
+#include <Log.hpp>
+#include <Protocol.hpp>
 
 using Poco::Net::HTMLForm;
 using Poco::Net::HTTPBasicCredentials;
@@ -54,11 +56,147 @@ using Poco::Util::Application;
 
 std::map<std::string, std::pair<std::string, std::string>> FileServerRequestHandler::FileHash;
 
+namespace {
+
+int functionConversation(int /*num_msg*/, const struct pam_message** /*msg*/,
+                         struct pam_response **reply, void *appdata_ptr)
+{
+    *reply = (struct pam_response *)malloc(sizeof(struct pam_response));
+    (*reply)[0].resp = strdup(static_cast<char *>(appdata_ptr));
+    (*reply)[0].resp_retcode = 0;
+
+    return PAM_SUCCESS;
+}
+
+/// Use PAM to check for user / password.
+bool isPamAuthOk(const std::string& userProvidedUsr, const std::string& userProvidedPwd)
+{
+    struct pam_conv localConversation { functionConversation, nullptr };
+    pam_handle_t *localAuthHandle = NULL;
+    int retval;
+
+    localConversation.appdata_ptr = const_cast<char *>(userProvidedPwd.c_str());
+
+    retval = pam_start("loolwsd", userProvidedUsr.c_str(), &localConversation, &localAuthHandle);
+
+    if (retval != PAM_SUCCESS)
+    {
+        LOG_ERR("pam_start returned " << retval);
+        return false;
+    }
+
+    retval = pam_authenticate(localAuthHandle, 0);
+
+    if (retval != PAM_SUCCESS)
+    {
+       if (retval == PAM_AUTH_ERR)
+       {
+           LOG_ERR("PAM authentication failure for user \"" << userProvidedUsr << "\".");
+       }
+       else
+       {
+           LOG_ERR("pam_authenticate returned " << retval);
+       }
+       return false;
+    }
+
+    LOG_INF("PAM authentication success for user \"" << userProvidedUsr << "\".");
+
+    retval = pam_end(localAuthHandle, retval);
+
+    if (retval != PAM_SUCCESS)
+    {
+        LOG_WRN("pam_end returned " << retval);
+    }
+
+    return true;
+}
+
+/// Check for user / password set in loolwsd.xml.
+bool isConfigAuthOk(const std::string& userProvidedUsr, const std::string& userProvidedPwd)
+{
+    const auto& config = Application::instance().config();
+    const std::string& user = config.getString("admin_console.username", "");
+
+    // Check for the username
+    if (user.empty())
+    {
+        LOG_ERR("Admin Console username missing, admin console disabled.");
+        return false;
+    }
+    else if (user != userProvidedUsr)
+    {
+        LOG_ERR("Admin Console wrong username.");
+        return false;
+    }
+
+    const char useLoolconfig[] = " Use loolconfig to configure the admin password.";
+
+    // do we have secure_password?
+    if (config.has("admin_console.secure_password"))
+    {
+        const std::string securePass = config.getString("admin_console.secure_password", "");
+        if (securePass.empty())
+        {
+            LOG_ERR("Admin Console secure password is empty, denying access." << useLoolconfig);
+            return false;
+        }
+
+#if HAVE_PKCS5_PBKDF2_HMAC
+        // Extract the salt from the config
+        std::vector<unsigned char> saltData;
+        std::vector<std::string> tokens = LOOLProtocol::tokenize(securePass, '.');
+        if (tokens.size() != 5 ||
+            tokens[0] != "pbkdf2" ||
+            tokens[1] != "sha512" ||
+            !Util::dataFromHexString(tokens[3], saltData))
+        {
+            LOG_ERR("Incorrect format detected for secure_password in config file." << useLoolconfig);
+            return false;
+        }
+
+        unsigned char userProvidedPwdHash[tokens[4].size() / 2];
+        PKCS5_PBKDF2_HMAC(userProvidedPwd.c_str(), -1,
+                          saltData.data(), saltData.size(),
+                          std::stoi(tokens[2]),
+                          EVP_sha512(),
+                          sizeof userProvidedPwdHash, userProvidedPwdHash);
+
+        std::stringstream stream;
+        for (unsigned long j = 0; j < sizeof userProvidedPwdHash; ++j)
+            stream << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(userProvidedPwdHash[j]);
+
+        // now compare the hashed user-provided pwd against the stored hash
+        return stream.str() == tokens[4];
+#else
+        const std::string pass = config.getString("admin_console.password", "");
+        LOG_ERR("The config file has admin_console.secure_password setting, "
+                << "but this application was compiled with old OpenSSL version, "
+                << "and this setting cannot be used." << (!pass.empty()? " Falling back to plain text password.": ""));
+
+        // careful, a fall-through!
+#endif
+    }
+
+    const std::string pass = config.getString("admin_console.password", "");
+    if (pass.empty())
+    {
+        LOG_ERR("Admin Console password is empty, denying access." << useLoolconfig);
+        return false;
+    }
+
+    return pass == userProvidedPwd;
+}
+
+}
+
 bool FileServerRequestHandler::isAdminLoggedIn(const HTTPRequest& request,
                                                HTTPResponse &response)
 {
+    assert(LOOLWSD::AdminEnabled);
+
     const auto& config = Application::instance().config();
-    const auto sslKeyPath = config.getString("ssl.key_file_path", "");
+    const std::string& sslKeyPath = config.getString("ssl.key_file_path", "");
 
     NameValueCollection cookies;
     request.getCookies(cookies);
@@ -80,76 +218,46 @@ bool FileServerRequestHandler::isAdminLoggedIn(const HTTPRequest& request,
         LOG_INF("No existing JWT cookie found");
     }
 
+    // If no cookie found, or is invalid, let the admin re-login
     HTTPBasicCredentials credentials(request);
-    std::string userProvidedPwd = credentials.getPassword();
+    const std::string& userProvidedUsr = credentials.getUsername();
+    const std::string& userProvidedPwd = credentials.getPassword();
 
-    // If no cookie found, or is invalid, let admin re-login
-    const std::string user = config.getString("admin_console.username", "");
-    std::string pass = config.getString("admin_console.password", "");
-
-    if (config.has("admin_console.secure_password"))
+    // Deny attempts to login without providing a username / pwd and fail right away
+    // We don't even want to allow a password-less PAM module to be used here,
+    // or anything.
+    if (userProvidedUsr.empty() || userProvidedPwd.empty())
     {
-#if HAVE_PKCS5_PBKDF2_HMAC
-        pass = config.getString("admin_console.secure_password");
-        // Extract the salt from the config
-        std::vector<unsigned char> saltData;
-        std::vector<std::string> tokens = LOOLProtocol::tokenize(pass, '.');
-        if (tokens.size() != 5 ||
-            tokens[0] != "pbkdf2" ||
-            tokens[1] != "sha512" ||
-            !Util::dataFromHexString(tokens[3], saltData))
-        {
-            LOG_ERR("Incorrect format detected for secure_password in config file."
-                    << "Denying access until correctly set."
-                    << "Use loolconfig to configure admin password.");
-            return false;
-        }
-
-        unsigned char userProvidedPwdHash[tokens[4].size() / 2];
-        PKCS5_PBKDF2_HMAC(userProvidedPwd.c_str(), -1,
-                          saltData.data(), saltData.size(),
-                          std::stoi(tokens[2]),
-                          EVP_sha512(),
-                          sizeof userProvidedPwdHash, userProvidedPwdHash);
-
-        std::stringstream stream;
-        for (unsigned long j = 0; j < sizeof userProvidedPwdHash; ++j)
-            stream << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(userProvidedPwdHash[j]);
-
-        userProvidedPwd = stream.str();
-        pass = tokens[4];
-#else
-        LOG_ERR("The config file has admin_console.secure_password setting, "
-                << "but this application was compiled with old OpenSSL version, "
-                << "and this setting cannot be used. Falling back to plain text password, if it is set.");
-#endif
-    }
-
-    if (user.empty() || pass.empty())
-    {
-        LOG_ERR("Admin Console credentials missing. Denying access until set.");
+        LOG_WRN("An attempt to log into Admin Console without username or password.");
         return false;
     }
 
-    if (credentials.getUsername() == user &&
-        userProvidedPwd == pass)
+    // Check if the user is allowed to use the admin console
+    if (config.getBool("admin_console.enable_pam", "false"))
     {
-        // generate and set the cookie
-        JWTAuth authAgent(sslKeyPath, "admin", "admin", "admin");
-        const std::string jwtToken = authAgent.getAccessToken();
-
-        Poco::Net::HTTPCookie cookie("jwt", jwtToken);
-        // bundlify appears to add an extra /dist -> dist/dist/admin
-        cookie.setPath("/loleaflet/dist/");
-        cookie.setSecure(LOOLWSD::isSSLEnabled() ||
-                         LOOLWSD::isSSLTermination());
-        response.addCookie(cookie);
-
-        return true;
+        // use PAM - it needs the username too
+        if (!isPamAuthOk(userProvidedUsr, userProvidedPwd))
+            return false;
+    }
+    else
+    {
+        // use the hash or password in the config file
+        if (!isConfigAuthOk(userProvidedUsr, userProvidedPwd))
+            return false;
     }
 
-    LOG_INF("Wrong admin credentials.");
-    return false;
+    // authentication passed, generate and set the cookie
+    JWTAuth authAgent(sslKeyPath, "admin", "admin", "admin");
+    const std::string jwtToken = authAgent.getAccessToken();
+
+    Poco::Net::HTTPCookie cookie("jwt", jwtToken);
+    // bundlify appears to add an extra /dist -> dist/dist/admin
+    cookie.setPath("/loleaflet/dist/");
+    cookie.setSecure(LOOLWSD::isSSLEnabled() ||
+                     LOOLWSD::isSSLTermination());
+    response.addCookie(cookie);
+
+    return true;
 }
 
 void FileServerRequestHandler::handleRequest(const HTTPRequest& request, Poco::MemoryInputStream& message,
@@ -158,6 +266,9 @@ void FileServerRequestHandler::handleRequest(const HTTPRequest& request, Poco::M
     try
     {
         bool noCache = false;
+#if ENABLE_DEBUG
+        noCache = true;
+#endif
         Poco::Net::HTTPResponse response;
         Poco::URI requestUri(request.getURI());
         LOG_TRC("Fileserver request: " << requestUri.toString());
@@ -190,10 +301,21 @@ void FileServerRequestHandler::handleRequest(const HTTPRequest& request, Poco::M
         {
             if (endPoint == "admin.html" ||
                 endPoint == "adminSettings.html" ||
+                endPoint == "adminHistory.html" ||
                 endPoint == "adminAnalytics.html")
+            {
+                preprocessAdminFile(request, socket);
+                return;
+            }
+
+            if (endPoint == "admin-bundle.js" ||
+                endPoint == "admin-localizations.js")
             {
                 noCache = true;
 
+                if (!LOOLWSD::AdminEnabled)
+                    throw Poco::FileAccessDeniedException("Admin console disabled");
+
                 if (!FileServerRequestHandler::isAdminLoggedIn(request, response))
                     throw Poco::Net::NotAuthenticatedException("Invalid admin login");
 
@@ -257,7 +379,7 @@ void FileServerRequestHandler::handleRequest(const HTTPRequest& request, Poco::M
             {
                 // Useful to not serve from memory sometimes especially during loleaflet development
                 // Avoids having to restart loolwsd everytime you make a change in loleaflet
-                const auto filePath = Poco::Path(LOOLWSD::FileServerRoot, relPath).absolute().toString();
+                const std::string filePath = Poco::Path(LOOLWSD::FileServerRoot, relPath).absolute().toString();
                 HttpHelper::sendFile(socket, filePath, mimeType, response, noCache);
                 return;
             }
@@ -339,7 +461,7 @@ void FileServerRequestHandler::readDirToHash(const std::string &basePath, const
     LOG_TRC("Pre-reading directory: " << basePath << path << "\n");
     workingdir = opendir((basePath + path).c_str());
 
-    while ((currentFile = readdir(workingdir)) != NULL)
+    while ((currentFile = readdir(workingdir)) != nullptr)
     {
         if (currentFile->d_name[0] == '.')
             continue;
@@ -362,7 +484,7 @@ void FileServerRequestHandler::readDirToHash(const std::string &basePath, const
             strm.opaque = Z_NULL;
             deflateInit2(&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 31, 8, Z_DEFAULT_STRATEGY);
 
-            auto buf = std::unique_ptr<char[]>(new char[fileStat.st_size]);
+            std::unique_ptr<char[]> buf(new char[fileStat.st_size]);
             std::string compressedFile;
             compressedFile.reserve(fileStat.st_size);
             std::string uncompressedFile;
@@ -440,10 +562,15 @@ std::string FileServerRequestHandler::getRequestPathname(const HTTPRequest& requ
     return path;
 }
 
+constexpr char BRANDING[] = "branding";
+#if ENABLE_SUPPORT_KEY
+constexpr char BRANDING_UNSUPPORTED[] = "branding-unsupported";
+#endif
+
 void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::MemoryInputStream& message, const std::shared_ptr<StreamSocket>& socket)
 {
     const auto host = ((LOOLWSD::isSSLEnabled() || LOOLWSD::isSSLTermination()) ? "wss://" : "ws://") + (LOOLWSD::ServerName.empty() ? request.getHost() : LOOLWSD::ServerName);
-    const auto params = Poco::URI(request.getURI()).getQueryParameters();
+    const Poco::URI::QueryParameters params = Poco::URI(request.getURI()).getQueryParameters();
 
     // Is this a file we read at startup - if not; its not for serving.
     const std::string relPath = getRequestPathname(request);
@@ -451,10 +578,10 @@ void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::
     std::string preprocess = *getUncompressedFile(relPath);
 
     HTMLForm form(request, message);
-    const std::string& accessToken = form.get("access_token", "");
-    const std::string& accessTokenTtl = form.get("access_token_ttl", "");
+    const std::string accessToken = form.get("access_token", "");
+    const std::string accessTokenTtl = form.get("access_token_ttl", "");
     LOG_TRC("access_token=" << accessToken << ", access_token_ttl=" << accessTokenTtl);
-    const std::string& accessHeader = form.get("access_header", "");
+    const std::string accessHeader = form.get("access_header", "");
     LOG_TRC("access_header=" << accessHeader);
 
     // Escape bad characters in access token.
@@ -490,12 +617,34 @@ void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::
     Poco::replaceInPlace(preprocess, std::string("%HOST%"), host);
     Poco::replaceInPlace(preprocess, std::string("%VERSION%"), std::string(LOOLWSD_VERSION_HASH));
 
+    static const std::string linkCSS("<link rel=\"stylesheet\" href=\"/loleaflet/" LOOLWSD_VERSION_HASH "/%s.css\">");
+    static const std::string scriptJS("<script src=\"/loleaflet/" LOOLWSD_VERSION_HASH "/%s.js\"></script>");
+
+    std::string brandCSS(Poco::format(linkCSS, std::string(BRANDING)));
+    std::string brandJS(Poco::format(scriptJS, std::string(BRANDING)));
+
     const auto& config = Application::instance().config();
+#if ENABLE_SUPPORT_KEY
+    const std::string keyString = config.getString("support_key", "");
+    SupportKey key(keyString);
+    if (!key.verify() || key.validDaysRemaining() <= 0)
+    {
+        brandCSS = Poco::format(linkCSS, std::string(BRANDING_UNSUPPORTED));
+        brandJS = Poco::format(scriptJS, std::string(BRANDING_UNSUPPORTED));
+    }
+#elif ENABLE_DEBUG
+    brandCSS = "";
+    brandJS = "";
+#endif
+
+    Poco::replaceInPlace(preprocess, std::string("<!--%BRANDING_CSS%-->"), brandCSS);
+    Poco::replaceInPlace(preprocess, std::string("<!--%BRANDING_JS%-->"), brandJS);
+
     const auto loleafletLogging = config.getString("loleaflet_logging", "false");
     Poco::replaceInPlace(preprocess, std::string("%LOLEAFLET_LOGGING%"), loleafletLogging);
-    const auto outOfFocusTimeoutSecs= config.getString("per_view.out_of_focus_timeout_secs", "60");
+    const std::string outOfFocusTimeoutSecs= config.getString("per_view.out_of_focus_timeout_secs", "60");
     Poco::replaceInPlace(preprocess, std::string("%OUT_OF_FOCUS_TIMEOUT_SECS%"), outOfFocusTimeoutSecs);
-    const auto idleTimeoutSecs= config.getString("per_view.idle_timeout_secs", "900");
+    const std::string idleTimeoutSecs= config.getString("per_view.idle_timeout_secs", "900");
     Poco::replaceInPlace(preprocess, std::string("%IDLE_TIMEOUT_SECS%"), idleTimeoutSecs);
 
     const std::string mimeType = "text/html";
@@ -544,8 +693,8 @@ void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::
 
     // Keep only the origin, reject everything else
     Poco::URI uriFrameAncestor(frameAncestor);
-    std::string frameAncestorScheme = uriFrameAncestor.getScheme();
-    std::string frameAncestorHost = uriFrameAncestor.getHost();
+    const std::string& frameAncestorScheme = uriFrameAncestor.getScheme();
+    const std::string& frameAncestorHost = uriFrameAncestor.getHost();
 
     if (!frameAncestor.empty() && Util::isValidURIScheme(frameAncestorScheme) && Util::isValidURIHost(frameAncestorHost))
     {
@@ -579,7 +728,7 @@ void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::
         bool keysPinned = false;
         while (config.has(pinPath))
         {
-            const auto pin = config.getString(pinPath, "");
+            const std::string pin = config.getString(pinPath, "");
             if (!pin.empty())
             {
                 hpkpOss << "pin-sha256=\"" << pin << "\"; ";
@@ -634,4 +783,57 @@ void FileServerRequestHandler::preprocessFile(const HTTPRequest& request, Poco::
     LOG_DBG("Sent file: " << relPath << ": " << preprocess);
 }
 
+void FileServerRequestHandler::preprocessAdminFile(const HTTPRequest& request,const std::shared_ptr<StreamSocket>& socket)
+{
+    Poco::Net::HTTPResponse response;
+
+    if (!LOOLWSD::AdminEnabled)
+        throw Poco::FileAccessDeniedException("Admin console disabled");
+
+    if (!FileServerRequestHandler::isAdminLoggedIn(request, response))
+        throw Poco::Net::NotAuthenticatedException("Invalid admin login");
+
+    static const std::string scriptJS("<script src=\"/loleaflet/dist/%s.js\"></script>");
+    static const std::string footerPage("<div class=\"footer navbar-fixed-bottom text-info text-center\"><strong>Key:</strong> %s &nbsp;&nbsp;<strong>Expiry Date:</strong> %s</div>");
+
+    const std::string relPath = getRequestPathname(request);
+    LOG_DBG("Preprocessing file: " << relPath);
+    std::string adminFile = *getUncompressedFile(relPath);
+    std::string brandJS(Poco::format(scriptJS, std::string(BRANDING)));
+    std::string brandFooter;
+
+#if ENABLE_SUPPORT_KEY
+    const auto& config = Application::instance().config();
+    const std::string keyString = config.getString("support_key", "");
+    SupportKey key(keyString);
+
+    if (!key.verify() || key.validDaysRemaining() <= 0)
+    {
+        brandJS = Poco::format(scriptJS, std::string(BRANDING_UNSUPPORTED));
+        brandFooter = Poco::format(footerPage, key.data(), Poco::DateTimeFormatter::format(key.expiry(), Poco::DateTimeFormat::RFC822_FORMAT));
+    }
+#elif ENABLE_DEBUG
+    brandJS = "";
+#endif
+
+    Poco::replaceInPlace(adminFile, std::string("<!--%BRANDING_JS%-->"), brandJS);
+    Poco::replaceInPlace(adminFile, std::string("<!--%FOOTER%-->"), brandFooter);
+
+    // Ask UAs to block if they detect any XSS attempt
+    response.add("X-XSS-Protection", "1; mode=block");
+    // No referrer-policy
+    response.add("Referrer-Policy", "no-referrer");
+    response.add("X-Content-Type-Options", "nosniff");
+    response.set("User-Agent", HTTP_AGENT_STRING);
+    response.set("Date", Poco::DateTimeFormatter::format(Poco::Timestamp(), Poco::DateTimeFormat::HTTP_FORMAT));
+
+    response.setContentType("text/html");
+    response.setChunkedTransferEncoding(false);
+
+    std::ostringstream oss;
+    response.write(oss);
+    oss << adminFile;
+    socket->send(oss.str());
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
